package com.persagy.account.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.persagy.account.manage.SaasAuthHandler;
import com.persagy.account.pojo.dto.SaasAccount;
import com.persagy.account.pojo.dto.SaasAccountProject;
import com.persagy.account.pojo.dto.SaasGroup;
import com.persagy.account.pojo.dto.SaasProject;
import com.persagy.account.pojo.vo.auth.SaasAccountProjectBatchVO;
import com.persagy.account.pojo.vo.auth.SaasAccountProjectCreateVO;
import com.persagy.account.pojo.vo.auth.SaasAccountProjectPageVO;
import com.persagy.account.pojo.vo.auth.SaasAccountProjectQueryVO;
import com.persagy.account.service.ISaasAccountProjectService;
import com.persagy.account.service.ISaasAccountService;
import com.persagy.account.service.ISaasGroupService;
import com.persagy.account.service.ISaasProjectService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;
import com.persagy.security.constant.CipherConstants;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 账号-集团项目关联信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */

@Api(tags = "账号-集团区域项目关联信息")
@RestController
@RequestMapping(value = "/accountProject", method = RequestMethod.POST)
public class SaasAccountProjectController {
	
	@Autowired
	private SaasAuthHandler saasAuthHandler;
	
    @Autowired
    private ISaasGroupService saasGroupService;

    @Autowired
    private ISaasAccountService saasAccountService;
    
    @Autowired
    private ISaasProjectService saasProjectService;
    
    @Autowired
    private ISaasAccountProjectService saasAccountProjectService;
    
    /**
     * 账号-集团项目普通列表查询
     */
    @ApiOperation(value = "账号-集团区域项目普通列表查询")
    @RequestMapping(value = "querySaasAccountProjectList")
    public ResponseResult querySaasAccountProjectList(@RequestBody @Validated SaasAccountProjectQueryVO queryVO) {
    	LambdaQueryWrapper<SaasAccountProject> queryWrapper = new SaasAccountProject.Builder().createQueryWrapper()
    			.groupCodeEq(queryVO.getGroupCode()).accountIdEq(queryVO.getAccountId()).areaIdEq(queryVO.getAreaId())
    			.projectIdEq(queryVO.getProjectId()).builderQueryWrapper();
    	List<SaasAccountProject> list = this.saasAccountProjectService.list(queryWrapper);
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 账号-集团项目数量统计列表分页查询
     */
    @ApiOperation(value = "账号-集团区域项目数量统计列表分页查询")
    @RequestMapping(value = "querAccountProjectTotal")
    public ResponseResult querAccountProjectTotal(@RequestBody @Validated SaasAccountProjectPageVO pageVO) {
    	SaasAccount saasAccount = new SaasAccount();
    	BeanUtils.copyProperties(pageVO, saasAccount);
    	saasAccount.setId(pageVO.getAccountId());
    	saasAccount.setValid(SaasCommonConstant.STATUS_1);
    	
    	// 先判断此账号的权限
    	Map<String, String> authMap = this.saasAuthHandler.validAccountAuth(pageVO.getAccountId(), pageVO.getGroupCode());
    	if (StringUtil.isNotBlank(authMap.get(CipherConstants.GROUP_CODE))) {
    		pageVO.setGroupCode(authMap.get(CipherConstants.GROUP_CODE));
		}
		String accountType = authMap.get(SaasCommonConstant.ACCOUNT_TYPE);
		if (SaasCommonConstant.STR_STATUS_0.equals(accountType)) {
			saasAccount.setId(null);
		}
    	Page<SaasAccount> pageList = this.saasAccountService.queryAccountPageList(saasAccount, pageVO.getPage(), pageVO.getSize());
    	if (pageList == null || CollectionUtil.isEmpty(pageList.getRecords())) {
    		return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
    	// 补充集团和项目信息
    	List<SaasAccount> records = pageList.getRecords();
    	SaasGroup saasGroup = new SaasGroup();
		saasGroup.setValid(SaasCommonConstant.STATUS_1);
    	for (SaasAccount account : records) {
    		account.setPassword(null);		// 剔除密码
    		// 超管，就是所有的集团和项目
    		if (SaasCommonConstant.STR_STATUS_0.equals(account.getAccountType())) {
				LambdaQueryWrapper<SaasGroup> queryWrapper = new SaasGroup.Builder().createQueryWrapper().validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
				queryWrapper.select(SaasGroup::getGroupCode, SaasGroup::getGroupName);
				List<SaasGroup> groups = this.saasGroupService.list(queryWrapper);
				account.setGroups(groups);
				
				LambdaQueryWrapper<SaasProject> wrapper = new SaasProject.Builder().createQueryWrapper().validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
				int count = this.saasProjectService.count(wrapper);
				account.setProjectNum(count);
			} else if (SaasCommonConstant.STR_STATUS_1.equals(account.getAccountType())) {
				// 业务超管
				LambdaQueryWrapper<SaasGroup> queryWrapper = new SaasGroup.Builder().createQueryWrapper()
						.groupCodeEq(account.getGroupCode()).validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
				queryWrapper.select(SaasGroup::getGroupCode, SaasGroup::getGroupName);
				List<SaasGroup> groups = this.saasGroupService.list(queryWrapper);
				account.setGroups(groups);
				
				LambdaQueryWrapper<SaasProject> wrapper = new SaasProject.Builder().createQueryWrapper()
						.groupCodeEq(account.getGroupCode()).validEq(SaasCommonConstant.STATUS_1).builderQueryWrapper();
				int count = this.saasProjectService.count(wrapper);
				account.setProjectNum(count);
			} else {
				// 普通账号，表关联查询
				List<SaasGroup> groups = this.saasGroupService.queryAllowGroupInfo(saasGroup, account.getId());
				account.setGroups(groups);
				
				int projectNum = this.saasProjectService.countAllowProjectNum(null, account.getId(), null);
				account.setProjectNum(projectNum);
			}
		}
        return ResponseResultUtil.successResult(records, pageList.getTotal());
    }
    
    /**
     * 账号-集团项目关联信息新增
     * 账号可见域，0-所有集团项目，1-单集团所有项目(此时，集团编码需存在)，2-其他
     */
	@ApiOperation(value = "新增账号-集团区域项目关联信息")
	@RequestMapping(value = "createSaasAccountProject")
    public ResponseResult createSaasAccountProject(@RequestBody @Validated SaasAccountProjectCreateVO createVO) {
		// 验证集团信息是否存在
		SaasGroup group = this.saasGroupService.getOne(createVO.getGroupCode());
		if (group == null) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "集团信息不存在");
		}
		// 验证项目信息是否存在
		if (StringUtil.isNotBlank(createVO.getProjectId())) {
			SaasProject saasProject = this.saasProjectService.getOne(createVO.getGroupCode(), createVO.getProjectId());
			if (saasProject == null) {
				return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "项目信息不存在");
			}
		}
		SaasAccountProject saasAccountProject = new SaasAccountProject();
		BeanUtils.copyProperties(createVO, saasAccountProject);
        boolean result = this.saasAccountProjectService.save(saasAccountProject);
        return result ? ResponseResultUtil.successResult("保存成功") : ResponseResultUtil.errorResult("保存失败");
    }
    
	
	/**
     * 账号-集团项目关联信息更新
     */
	@ApiOperation(value = "删除新增账号-集团区域项目关联信息")
	@RequestMapping(value = "resetSaasAccountProject")
    public ResponseResult resetSaasAccountProject(@RequestBody @Validated SaasAccountProjectBatchVO batchVO) {
		boolean result = this.saasAccountProjectService.resetSaasAccountProject(batchVO.getAccountId(), batchVO.getAuths());
        return result ? ResponseResultUtil.successResult("重置成功") : ResponseResultUtil.errorResult("重置失败");
    }
	
    /**
     * 账号-集团项目关联信息删除
     */
	@ApiOperation(value = "删除账号-集团区域项目关联信息")
	@RequestMapping(value = "deleteSaasAccountProject")
    public ResponseResult deleteSaasAccountProject(@RequestBody @Validated SaasAccountProjectQueryVO queryVO) {
		LambdaQueryWrapper<SaasAccountProject> updateWrapper = new SaasAccountProject.Builder().createQueryWrapper()
				.accountIdEq(queryVO.getAccountId()).groupCodeEq(queryVO.getGroupCode()).areaIdEq(queryVO.getAreaId())
				.projectIdEq(queryVO.getProjectId()).builderQueryWrapper();
		boolean result = this.saasAccountProjectService.remove(updateWrapper);
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }

}
