package com.persagy.account.pojo.dto;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.persagy.common.utils.StringUtil;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 账号-集团项目关联信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName("saas_account_project")
public class SaasAccountProject extends Model<SaasAccountProject> {
	private static final long serialVersionUID = 5519395457650465434L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String groupCode; // 集团编码
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String accountId; // 账号ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String areaId; // 项目ID
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String projectId; // 项目ID
	
	public static class Builder {

		private LambdaQueryWrapper<SaasAccountProject> queryWrapper = null;

		private LambdaUpdateWrapper<SaasAccountProject> updateWrapper = null;

		public Builder createQueryWrapper() {
			queryWrapper = new LambdaQueryWrapper<SaasAccountProject>();
			return this;
		}

		public Builder createUpdateWrapper() {
			updateWrapper = new LambdaUpdateWrapper<SaasAccountProject>();
			return this;
		}

		public Builder groupCodeEq(String groupCode) {
			if (StringUtil.isNotBlank(groupCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountProject::getGroupCode, groupCode);
				} else {
					queryWrapper.eq(SaasAccountProject::getGroupCode, groupCode);
				}
			}
			return this;
		}

		public Builder accountIdEq(String accountId) {
			if (StringUtil.isNotBlank(accountId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountProject::getAccountId, accountId);
				} else {
					queryWrapper.eq(SaasAccountProject::getAccountId, accountId);
				}
			}
			return this;
		}

		public Builder areaIdEq(String areaId) {
			if (StringUtil.isNotBlank(areaId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountProject::getAreaId, areaId);
				} else {
					queryWrapper.eq(SaasAccountProject::getAreaId, areaId);
				}
			}
			return this;
		}
		
		public Builder projectIdEq(String projectId) {
			if (StringUtil.isNotBlank(projectId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountProject::getProjectId, projectId);
				} else {
					queryWrapper.eq(SaasAccountProject::getProjectId, projectId);
				}
			}
			return this;
		}

		public LambdaQueryWrapper<SaasAccountProject> builderQueryWrapper() {
			return queryWrapper;
		}

		public LambdaUpdateWrapper<SaasAccountProject> builderUpdateWrapper() {
			return updateWrapper;
		}
	}
}
