package com.persagy.account.pojo.dto;

import java.util.Date;
import java.util.Set;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import com.persagy.common.annotation.IdGeneratorPrefix;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.model.BaseEntity;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 项目信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName(value = "saas_project", autoResultMap = true)
@IdGeneratorPrefix(SaasCommonConstant.SAAS_PROJECT_ID_PREFIX)
public class SaasProject extends BaseEntity<SaasProject> {
	private static final long serialVersionUID = -4224325454752727633L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String groupCode; // 集团编码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String projectId; // 项目ID，与集团编码联合唯一，业务保证

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String areaId; // 项目所属区域，必须为最底层的区域

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String projectLocalId; // 项目本地编码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String projectName; // 项目名称

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String provinceCode;	//项目所在省

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String cityCode;	//项目所在市
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String zoneCode;	//项目所在区
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String location;	//项目详细地址
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date openTime;	//项目开业时间
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String projectFun;		//项目功能
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String projectWeather;	//项目天气
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY, typeHandler = FastjsonTypeHandler.class)
	private Set<String> projectImage;	//项目图标
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String longitude;		//项目经度
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String latitude;		//项目纬度
	
	@TableField(fill = FieldFill.INSERT, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date createTime; 	// 创建时间

	@TableField(fill = FieldFill.INSERT_UPDATE, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date updateTime; 	// 更新时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String updateUser; 	// 最后一次操作者ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String remark; 		// 备注

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Integer valid; 		// 0-不可用，1-可用

	@TableField(exist = false)
	private Set<String> projectIds; 	// 项目ID集合
	
	@TableField(exist = false)
	private String accountId; 	// 当前账号ID
	
	@TableField(exist = false)
	private String areaName; // 区域名称
	
	public static class Builder {

		private LambdaQueryWrapper<SaasProject> queryWrapper = null;

		private LambdaUpdateWrapper<SaasProject> updateWrapper = null;

		public Builder createQueryWrapper() {
			queryWrapper = new LambdaQueryWrapper<SaasProject>();
			return this;
		}

		public Builder createUpdateWrapper() {
			updateWrapper = new LambdaUpdateWrapper<SaasProject>();
			return this;
		}

		public Builder idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getId, id);
				} else {
					queryWrapper.eq(SaasProject::getId, id);
				}
			}
			return this;
		}

		public Builder groupCodeEq(String groupCode) {
			if (StringUtil.isNotBlank(groupCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getGroupCode, groupCode);
				} else {
					queryWrapper.eq(SaasProject::getGroupCode, groupCode);
				}
			}
			return this;
		}

		public Builder projectIdEq(String projectId) {
			if (StringUtil.isNotBlank(projectId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProjectId, projectId);
				} else {
					queryWrapper.eq(SaasProject::getProjectId, projectId);
				}
			}
			return this;
		}
		
		public Builder projectIdIn(Set<String> projectIds) {
			if (CollectionUtil.isNotEmpty(projectIds)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProjectId, projectIds);
				} else {
					queryWrapper.eq(SaasProject::getProjectId, projectIds);
				}
			}
			return this;
		}

		public Builder areaIdEq(String areaId) {
			if (StringUtil.isNotBlank(areaId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getAreaId, areaId);
				} else {
					queryWrapper.eq(SaasProject::getAreaId, areaId);
				}
			}
			return this;
		}

		public Builder projectLocalIdEq(String projectLocalId) {
			if (StringUtil.isNotBlank(projectLocalId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProjectLocalId, projectLocalId);
				} else {
					queryWrapper.eq(SaasProject::getProjectLocalId, projectLocalId);
				}
			}
			return this;
		}

		public Builder projectNameEq(String projectName) {
			if (StringUtil.isNotBlank(projectName)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProjectName, projectName);
				} else {
					queryWrapper.eq(SaasProject::getProjectName, projectName);
				}
			}
			return this;
		}
		
		public Builder projectNameLike(String projectName) {
			if (StringUtil.isNotBlank(projectName)) {
				if (updateWrapper != null) {
					updateWrapper.like(SaasProject::getProjectName, projectName);
				} else {
					queryWrapper.like(SaasProject::getProjectName, projectName);
				}
			}
			return this;
		}

		public Builder provinceCodeEq(String provinceCode) {
			if (StringUtil.isNotBlank(provinceCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProvinceCode, provinceCode);
				} else {
					queryWrapper.eq(SaasProject::getProvinceCode, provinceCode);
				}
			}
			return this;
		}
		
		public Builder cityCodeEq(String cityCode) {
			if (StringUtil.isNotBlank(cityCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getCityCode, cityCode);
				} else {
					queryWrapper.eq(SaasProject::getCityCode, cityCode);
				}
			}
			return this;
		}
		
		public Builder zoneCodeEq(String zoneCode) {
			if (StringUtil.isNotBlank(zoneCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getZoneCode, zoneCode);
				} else {
					queryWrapper.eq(SaasProject::getZoneCode, zoneCode);
				}
			}
			return this;
		}
		
		public Builder openTimeEq(Date openTime) {
			if (null != openTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getOpenTime, openTime);
				} else {
					queryWrapper.eq(SaasProject::getOpenTime, openTime);
				}
			}
			return this;
		}
		
		public Builder openTimeGe(Date openTime) {
			if (null != openTime) {
				if (updateWrapper != null) {
					updateWrapper.ge(SaasProject::getOpenTime, openTime);
				} else {
					queryWrapper.ge(SaasProject::getOpenTime, openTime);
				}
			}
			return this;
		}
		
		public Builder projectFunEq(String projectFun) {
			if (StringUtil.isNotBlank(projectFun)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProjectFun, projectFun);
				} else {
					queryWrapper.eq(SaasProject::getProjectFun, projectFun);
				}
			}
			return this;
		}
		
		public Builder projectWeatherEq(String projectWeather) {
			if (StringUtil.isNotBlank(projectWeather)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getProjectWeather, projectWeather);
				} else {
					queryWrapper.eq(SaasProject::getProjectWeather, projectWeather);
				}
			}
			return this;
		}
		
		public Builder createTimeEq(Date createTime) {
			if (null != createTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getCreateTime, createTime);
				} else {
					queryWrapper.eq(SaasProject::getCreateTime, createTime);
				}
			}
			return this;
		}

		public Builder updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getUpdateTime, updateTime);
				} else {
					queryWrapper.eq(SaasProject::getUpdateTime, updateTime);
				}
			}
			return this;
		}

		public Builder updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getUpdateUser, updateUser);
				} else {
					queryWrapper.eq(SaasProject::getUpdateUser, updateUser);
				}
			}
			return this;
		}

		public Builder remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getRemark, remark);
				} else {
					queryWrapper.eq(SaasProject::getRemark, remark);
				}
			}
			return this;
		}

		public Builder validEq(Integer valid) {
			if (null != valid) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasProject::getValid, valid);
				} else {
					queryWrapper.eq(SaasProject::getValid, valid);
				}
			}
			return this;
		}
		
		public Builder last(String lastSql) {
			if (StringUtil.isNotBlank(lastSql)) {
				if (updateWrapper != null) {
					updateWrapper.last(lastSql);
				} else {
					queryWrapper.last(lastSql);
				}
			}
			return this;
		}

		public LambdaQueryWrapper<SaasProject> builderQueryWrapper() {
			return queryWrapper;
		}

		public LambdaUpdateWrapper<SaasProject> builderUpdateWrapper() {
			return updateWrapper;
		}
	}
}
