package com.persagy.account.pojo.vo.menu;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import com.alibaba.fastjson.JSONObject;
import com.persagy.common.model.BaseUserModel;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 菜单信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ToString
@ApiModel
public class SaasMenuCreateVO extends BaseUserModel {

	@ApiModelProperty(value = "菜单ID，更新时不可为空", example = "123456")
	private String id; // 菜单ID
	
	@ApiModelProperty(value = "上级菜单ID", example = "123456")
	private String parentId; // 上级菜单ID

	@ApiModelProperty(value = "产品ID", example = "PD1001")
	private String productId; // 产品ID
	
	//@NotBlank(message = "菜单编码，不可为空")
	@ApiModelProperty(value = "菜单编码", example = "ceshi", required = true)
	private String menuCode; // 菜单编码
	
	//@NotBlank(message = "菜单名称，不可为空")
	@ApiModelProperty(value = "菜单名称", example = "测试菜单", required = true)
	private String menuName; // 菜单名称

	@ApiModelProperty(value = "菜单URL", example = "/account/query")
	private String menuUrl;  // 菜单URL

	@ApiModelProperty(value = "URL类型，0-相对路径，1-绝对路径", example = "0")
	private String urlType;	//URL类型，0-相对路径，1-绝对路径
	
	//@NotBlank(message = "菜单图标，不可为空")
	@ApiModelProperty(value = "菜单图标", example = "http://127.0.0.1:8080/test.img")
	private String menuIcon; // 菜单图标
	
	@Min(value = 1, message = "菜单顺序，不可小于1")
	@NotNull(message = "菜单顺序，不可为空")
	@ApiModelProperty(value = "菜单顺序", example = "1", required = true)
	private Integer menuSort; // 菜单顺序

	@NotBlank(message = "菜单类型，不可为空")
	@ApiModelProperty(value = "菜单类型，0-系统菜单，1-业务菜单", example = "0")
	private String menuType;	//菜单类型，0-系统菜单，1-业务菜单
	
	@ApiModelProperty(value = "分组菜单，0-非分组菜单，1-分组菜单", example = "0")
	private String menuGroup;	// 分组菜单，0-非分组菜单，1-分组菜单
	
	@ApiModelProperty(value = "菜单额外字段，json格式", example = "0")
	private JSONObject extraMenu;	// 菜单额外字段
	
	@ApiModelProperty(value = "备注", example = "测试菜单")
	private String remark;   // 备注

}
