package com.persagy.account.service.impl;

import java.util.Date;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.account.mapper.SaasGroupProductPageMapper;
import com.persagy.account.mapper.SaasGroupProductMapper;
import com.persagy.account.mapper.SaasProjectProductMapper;
import com.persagy.account.pojo.dto.SaasGroupProduct;
import com.persagy.account.pojo.dto.SaasGroupProductPage;
import com.persagy.account.pojo.dto.SaasProjectProduct;
import com.persagy.account.service.ISaasGroupProductService;
import com.persagy.common.service.impl.SuperServiceImpl;

/**
 * 集团签约产品
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-05-07 12:02:40
 */
@Service
public class SaasGroupProductServiceImpl extends SuperServiceImpl<SaasGroupProductMapper, SaasGroupProduct> implements ISaasGroupProductService {

	@Autowired
	private SaasProjectProductMapper saasProjectProductMapper;
	
	@Autowired
	private SaasGroupProductPageMapper saasGroupProductPageMapper;
	
	@Override
	@Transactional
	public boolean createSaasGroupProduct(SaasGroupProduct saasGroupProduct, Set<String> pageIds) {
		// 优先创建产品集团签约信息
		saasGroupProduct.setProductContract(new Date());
		Integer result = this.baseMapper.insert(saasGroupProduct);
		if (SqlHelper.retBool(result)) {
			// 创建签约信息与产品页面的关联关系
			result = this.saasGroupProductPageMapper.insertSaasGroupProductPage(saasGroupProduct.getId(), pageIds);
		}
		return SqlHelper.retBool(result);
	}

	@Override
	@Transactional
	public boolean updateSaasGroupProduct(SaasGroupProduct saasGroupProduct, Set<String> pageIds) {
		// 优先更新产品集团签约信息
		Integer result = this.baseMapper.updateById(saasGroupProduct);
		if (SqlHelper.retBool(result)) {
			// 删除签约信息与产品页面的关联关系
			LambdaQueryWrapper<SaasGroupProductPage> queryWrapper = new SaasGroupProductPage.BuilderQueryWrapper().contractIdEq(saasGroupProduct.getId()).builder();
			this.saasGroupProductPageMapper.delete(queryWrapper);
			// 创建签约信息与产品页面的关联关系
			result = this.saasGroupProductPageMapper.insertSaasGroupProductPage(saasGroupProduct.getId(), pageIds);
		}
		return SqlHelper.retBool(result);
	}

	@Override
	@Transactional
	public boolean deleteSaasGroupProduct(String contractId) {
		Integer result = this.baseMapper.deleteById(contractId);
		if (SqlHelper.retBool(result)) {
			// 删除签约信息与产品页面的关联关系
			LambdaQueryWrapper<SaasGroupProductPage> pageWrapper = new SaasGroupProductPage.BuilderQueryWrapper().contractIdEq(contractId).builder();
			result = this.saasGroupProductPageMapper.delete(pageWrapper);
			// 删除项目签约的产品信息
			LambdaQueryWrapper<SaasProjectProduct> queryWrapper = new SaasProjectProduct.BuilderQueryWrapper().contractIdEq(contractId).builder();
			this.saasProjectProductMapper.delete(queryWrapper);
		}
		return SqlHelper.retBool(result);
	}

}
