package com.persagy.account.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.account.mapper.SaasMenuMapper;
import com.persagy.account.pojo.dto.SaasMenu;
import com.persagy.account.pojo.vo.menu.SaasMenuSort;
import com.persagy.account.service.ISaasMenuService;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.common.utils.StringUtil;

/**
 * 菜单信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Service
public class SaasMenuServiceImpl extends SuperServiceImpl<SaasMenuMapper, SaasMenu> implements ISaasMenuService {
	
	@Override
	public List<SaasMenu> querySaasMenuTree(String accountId, String groupCode, String appId, String menuId, Integer valid) {
		return this.baseMapper.querySaasMenuTree(accountId, groupCode, appId, menuId, valid);
	}

	@Override
	public List<SaasMenu> queryMenuAndEmptyFunction(String groupCode, String roleId) {
		return this.baseMapper.queryMenuAndEmptyFunction(groupCode, roleId);
	}

	@Override
	@Transactional
	public boolean sortSaasMenu(List<SaasMenuSort> menus) {
		Integer result = this.baseMapper.sortSaasMenu(menus);
		return SqlHelper.retBool(result);
	}

	@Override
	public boolean validMenuCode(String appId, String menuType, String menuCode, String menuId) {
		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().appIdEq(appId).menuTypeEq(menuType).menuCodeEq(menuCode).builderQueryWrapper();
		SaasMenu selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne != null && (StringUtil.isBlank(menuId) || !menuId.equals(selectOne.getId()))) {
			return false;
		}
		return true;
	}
	
	@Override
	public boolean validMenuName(String appId, String menuType, String menuName, String menuId) {
		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().appIdEq(appId).menuTypeEq(menuType).menuNameEq(menuName).builderQueryWrapper();
		SaasMenu selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne != null && (StringUtil.isBlank(menuId) || !menuId.equals(selectOne.getId()))) {
			return false;
		}
		return true;
	}

}
