package com.persagy.sso.client;

import com.persagy.common.utils.ResponseResult;
import com.persagy.sso.client.fallback.SaasAccountClientFallbackFactory;
import com.persagy.sso.config.FeignConfig;
import com.persagy.sso.constant.FeignClientNameConstant;
import com.persagy.sso.pojo.dto.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;

import java.util.List;

/**
 * @Description: 调用fm-person的接口信息
 * @Author: xgq
 * @Date: 2021/4/14 21:51
 * @return: null
 **/
@FeignClient(name = FeignClientNameConstant.SAAS_ACCOUNT_CLIENT_NAME,
        url=FeignClientNameConstant.SAAS_ACCOUNT_CLIENT_URL ,fallbackFactory = SaasAccountClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface SaasAccountClient {

    /**
     * @Description: 根据条件查询账号信息
     * @Author: xgq
     * @Date: 2021/4/16 0:49
     * @param accountDTO:
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/account/querySaasAccountByUsername")
    ResponseResult queryAccountByCondition(@RequestBody  AccountDTO accountDTO);

    /**
     * @Description: 根据账号查询角色
     * @Author: xgq
     * @Date: 2021/4/16 1:45
     * @param queryRoleDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/accountRole/querySaasRoleMenuList")
    ResponseResult queryRoleByAccount(@RequestBody QueryRoleDTO queryRoleDTO,@RequestHeader("token") String token);

    /**
     * @Description: 更新账号
     * @Author: xgq
     * @Date: 2021/4/16 2:27
     * @param createDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/account/updateSaasAccount")
    ResponseResult updateAccount(@RequestBody SaasAccountCreateDTO createDTO,@RequestHeader("token") String token);


    /*
     * @Description: 新增账号信息
     * @Author: xgq
     * @Date: 2021/4/16 9:54
     * @param createDTO:
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/account/createSaasAccount")
    ResponseResult insertAccount(@RequestBody SaasAccountCreateDTO createDTO,@RequestHeader("token") String token);

    /**
     * @Description: 根据角色名称查询角色
     * @Author: xgq
     * @Date: 2021/4/17 19:58
     * @param queryJobDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    @PostMapping("/role/querySaasRoleByName")
    ResponseResult queryJobBuCondition(@RequestBody QueryJobDTO queryJobDTO,@RequestHeader("token") String token);

    @PostMapping("/account/querySaasAccountList")
    ResponseResult queryAccountListBySource(@RequestBody  AccountDTO accountDTO,@RequestHeader("token")  String token);

    @PostMapping("/account/deleteSaasAccountListByIds")
    ResponseResult deleteByIds(@RequestBody AccountDTO accountDTO, @RequestHeader("token") String token);
}
