package com.persagy.sso.manage;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.common.exception.BusinessException;
import com.persagy.common.utils.ResponseResult;
import com.persagy.sso.constant.BaseConstant;
import com.persagy.sso.pojo.dto.*;
import com.persagy.sso.service.*;
import com.persagy.sso.util.BaseUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @ClassName : SynUserManage
 * @Author ：xgq
 * @Date ：Created in 2021/4/14 23:09
 * @Description： TODO
 * @Version : 1.0
 */
@Component
public class SynUserManage {



    @Autowired
    private IDeptReflexService deptReflexService;
    @Autowired
    private IJobReflexService jobReflexService;
    @Autowired
    private IAccountReflexService accountReflexService;
    @Autowired
    private IDeptService deptService;
    @Autowired
    private IJobService jobService;
    @Autowired
    private IPersonService personService;
    @Autowired
    private IAccountService accountService;
    @Autowired
    private IUserService userService;
    @Autowired
    private IWdAccountService wdAccountService;

    /***
     * @Description: 保存部门信息
     * @Author: xgq
     * @Date: 2021/4/13 23:11
     * @param oaDeptId
     * @param addDepartmentDTO :
     * @param deptType
     * @param token
     * @return: void
     **/
    @Transactional
    public String saveGroupDept(String oaDeptId, AddDepartmentDTO addDepartmentDTO, String deptType, String token) {
        //还没有，创建一个集团部门---父id是0，只有一个，第一次同步创建
        //顶级部门，先往部门表添加一条数据
        if(BaseConstant.DEPT_TYPE_GROUP.equals(deptType)){
            addDepartmentDTO.setParentId(BaseConstant.DEPT_TYPE_GROUP);
            addDepartmentDTO.setType(BaseConstant.DEPT_TYPE_GROUP);
            addDepartmentDTO.setName(BaseConstant.GROUP_NAME);
        }
        addDepartmentDTO.setResourceFrom(BaseConstant.RESOURCE_FROM_OA_WD);
        ResponseResult result = this.deptService.saveDept(addDepartmentDTO,token);
        if (!BaseUtils.CheckResult(result)) {
            throw new BusinessException("save group dept error:"+BaseUtils.getMsg(result) );
        }
        JSONObject content = BaseUtils.getContent(result);
        String deptId = content.getString(BaseConstant.RESPONSE_ID);
        if(StringUtils.isBlank(deptId)){
            throw new BusinessException("save group dept error: no id info..." );
        }
        //创建一条映射数据
        DeptReflex deptReflex = new DeptReflex();
        deptReflex.setId(BaseUtils.getUuid());
        deptReflex.setOaDeptId(oaDeptId);
         deptReflex.setDeptId(deptId);
        boolean save = deptReflexService.save(deptReflex);
        if(!save){
            throw new BusinessException("save  dept reflex error..." );
        }
        return deptId;
    }


    /**
     * @Description: 保存岗位信息
     * @Author: xgq
     * @Date: 2021/4/14 14:50
     * @return: void
     * @param addJobDTO
     * @param jobReflex
     * @param token    */
    public String saveJob(AddJobDTO addJobDTO, JobReflex jobReflex, String token) {
        //保存岗位信息
        ResponseResult result = this.jobService.saveJob(addJobDTO,token);
        if(!BaseUtils.CheckResult(result)){
            throw new RuntimeException("save job info error:"+BaseUtils.getMsg(result) );
        }
        //获取返回的岗位的id
        JSONObject content = BaseUtils.getContent(result);
       String jobId = content.getString(BaseConstant.RESPONSE_ID);
        if(StringUtils.isBlank(jobId)){
          throw new BusinessException("save  job error: no id info..." );
        }
        jobReflex.setId(BaseUtils.getUuid());
        jobReflex.setJobId(jobId);
        boolean saveJob = jobReflexService.save(jobReflex);
        if(!saveJob){
            throw new RuntimeException("save  job reflex error..." );
        }
        return jobId;
    }

    /**
     * @Description: 更新岗位信息
     * @Author: xgq
     * @Date: 2021/4/15 1:45
     * @param addJobDTO :
     * @param token
     * @return: void
     **/
    public ResponseResult updateJob(AddJobDTO addJobDTO, String token) {
         return  jobService.updateJob(addJobDTO,token);
    }

    /**
     * @Description: 保存人员信息
     * @Author: xgq
     * @Date: 2021/4/14 15:57
     * @param addPersonDTO
     * @param user :
     * @param deptId
     * @param jobId
     * @param accountId
     * @param token
     * @return: void
     **/
    public ResponseResult savePerson(AddPersonDTO addPersonDTO, WdUser user, String deptId, String jobId, String accountId, String token) {
        addPersonDTO.setJobNumber(user.getEmployeeCode());
        addPersonDTO.setName(user.getEmployeeName());
        //看看传的是中文还是数字
        //  addPersonDTO.setGender(user.getGender());
        addPersonDTO.setGender(1);
        addPersonDTO.setUsername(user.getUsername());
        //部门
        addPersonDTO.setDepIds(Arrays.asList(deptId));
        //主岗
        addPersonDTO.setMainDuty(jobId);
        //账号id
        addPersonDTO.setAccountId(accountId);
        //有用户id就更新，没有就新建
         if(StringUtils.isBlank(addPersonDTO.getId())){
             return personService.savePerson(addPersonDTO,token);
         }else {
             return personService.updatePerson(addPersonDTO,token);
         }
    }

    /**
     * @Description: 根据条件查询部门信息
     * @Author: xgq
     * @Date: 2021/4/15 19:51
     * @param queryDeptDTO :
     * @param token
     * @return: void
     **/
    public ResponseResult queryDeptByCondition(QueryDeptDTO queryDeptDTO, String token) {
        return deptService.queryDeptByCondition(queryDeptDTO,token);
    }

    /**
     * @Description: 更新部门信息
     * @Author: xgq
     * @Date: 2021/4/16 0:38
     * @param addDepartmentDTO :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    public ResponseResult updateDept(AddDepartmentDTO addDepartmentDTO, String token) {
        return deptService.updateDept(addDepartmentDTO,token);
    }


    /**
     * @Description: 查询账号是否存在
     * @Author: xgq
     * @Date: 2021/4/16 0:39
     * @param username :
     * @param token
     * @return: java.lang.Boolean
     **/
    public JSONObject queryAccountByCondition(String username, String token) {
        AccountDTO accountDTO = new AccountDTO();
        accountDTO.setUsername(username);
       accountDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
    ResponseResult result  =   accountService.queryAccountByCondition(accountDTO);
        if(!BaseUtils.CheckResult(result)){
            throw new BusinessException("query account info error:"+BaseUtils.getMsg(result) );
        }
        JSONArray array = BaseUtils.getContent2Array(result);
        if(null == array || (null !=array &&  array.size()==0)){
            return null;
        } else {
            return array.getJSONObject(0);
        }
    }





    /**
     * @Description: 根据账号查询角色
     * @Author: xgq
     * @Date: 2021/4/16 1:38
     * @param oldAccountId :
     * @param token
     * @return: void
     **/
    public String queryRoleByAccount(String oldAccountId, String token) {
        QueryRoleDTO queryRoleDTO = new QueryRoleDTO();
        queryRoleDTO.setAccountBelong(BaseConstant.ACCOUNT_TYPE_BUSINESS);
         queryRoleDTO.setAppId(BaseConstant.APP_ID_PC);
         queryRoleDTO.setAccountIds(Arrays.asList(oldAccountId));
        ResponseResult result = accountService.queryRoleByAccount(queryRoleDTO,token);
         if(!BaseUtils.CheckResult(result)){
           //出现错误，直接更新
             return null;
         }
        JSONArray array = BaseUtils.getContent2Array(result);
         if(null ==array){
              return null;
         }
        if(array.size()==0){
             return null;
        }
        JSONObject jsonObject = array.getJSONObject(0);
         if(null == jsonObject){
             return null;
         }
        return  jsonObject.getString(BaseConstant.ROLE_ID);
    }

    /**
     * @Description: 更新账号的角色信息
     * @Author: xgq
     * @Date: 2021/4/16 2:00
     * @param oldAccountId :
     * @param username :
     * @param jobId :
     * @param token
     * @return: void
     **/
    public ResponseResult updateAccountInfo(String oldAccountId, String username, String jobId, String token) {
        SaasAccountCreateDTO createDTO = new SaasAccountCreateDTO();
        createDTO.setAccountBelong(BaseConstant.ACCOUNT_TYPE_BUSINESS);
        createDTO.setAccountId(oldAccountId);
        createDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
        createDTO.setAppId(BaseConstant.APP_ID_PC);
        createDTO.setUsername(username);
        createDTO.setAccountType(BaseConstant.ACCOUNT_TYPE_NORMAL);
        createDTO.setAccountSource(BaseConstant.GROUP_CODE_WD);
        //账号有效期，0-期内有效，1-永久有效
        createDTO.setValidLast(BaseConstant.VALID_TRUE);
        createDTO.setValid(Integer.valueOf(BaseConstant.VALID_TRUE));
        createDTO.setCasType(BaseConstant.CASE_TYPE_MAJOR);
        AccountDTO accountDTO = new AccountDTO();
        accountDTO.setRoleId(jobId);
        createDTO.setRoles(Arrays.asList(accountDTO));
      return   accountService.updateAccount(createDTO,token);
    }

    /**
     * @Description: 保存账号信息
     * @Author: xgq
     * @Date: 2021/4/16 9:46
     * @param user :
     * @param jobId :
     * @param token
     * @return: void
     **/
    public String saveAccountInfo(WdUser user, String jobId, String token) {
        //保存账号信息
        SaasAccountCreateDTO createDTO = new SaasAccountCreateDTO();
        createDTO.setAccountBelong(BaseConstant.ACCOUNT_TYPE_BUSINESS);
        createDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
        createDTO.setAppId(BaseConstant.APP_ID_PC);
        createDTO.setUsername(user.getUsername());
        createDTO.setAccountType(BaseConstant.ACCOUNT_TYPE_NORMAL);
        //accountSource：万达：201
        createDTO.setAccountSource(BaseConstant.ACCOUNT_SOURCE_WD);
        //账号有效期，0-期内有效，1-永久有效
        createDTO.setValidLast(BaseConstant.VALID_TRUE);
        createDTO.setValid(Integer.valueOf(BaseConstant.VALID_TRUE));
        createDTO.setCasType(BaseConstant.CASE_TYPE_MAJOR);
        AccountDTO accountDTO = new AccountDTO();
        accountDTO.setRoleId(jobId);
        createDTO.setRoles(Arrays.asList(accountDTO));
      ResponseResult result =   accountService.insertAccount(createDTO,token);
        if (!BaseUtils.CheckResult(result)) {
            throw new RuntimeException("save account info error:"+BaseUtils.getMsg(result) );
        }
        //账号id
        String accountId = BaseUtils.getContentStr(result);
        //保存映射关系
        AccountReflex accountReflex = new AccountReflex();
        accountReflex.setId(BaseUtils.getUuid());
        accountReflex.setOaAccountId(user.getEmployeeID());
        accountReflex.setAccountId(accountId);
        boolean save = accountReflexService.save(accountReflex);
        if(save){
            return accountId;
        } else {
            throw new BusinessException("save account reflex info error..." );
        }
    }

    /**
     * @Description: 根据username查询人是否存在
     * @Author: xgq
     * @Date: 2021/4/16 15:40
     * @param user :
     * @param token
     * @return: com.persagy.common.utils.ResponseResult
     **/
    public JSONObject queryPersonByCondition(WdUser user, String token) {
        QueryPersonDTO queryPersonDTO = new QueryPersonDTO();
        String username = user.getUsername();
        queryPersonDTO.setUsername(username);
        ResponseResult result =   personService.queryPersonByCondition(queryPersonDTO,token);
        if(!BaseUtils.CheckResult(result)){
            throw new BusinessException("query user by username  error: " +BaseUtils.getMsg(result));
        }
        JSONObject records = BaseUtils.getContent(result);
        JSONArray userArray = records.getJSONArray(BaseConstant.RESPONSE_RECORDS);
        if(null == userArray){
            throw new BusinessException("query user by username  error: no records info... ");
        }
        if(userArray.size()==0){
            return null;
        }
        return userArray.getJSONObject(0);
    }

    /**
     * @Description: 根据角色名称查询角色信息
     * @Author: xgq
     * @Date: 2021/4/17 19:54
     * @param queryJobDTO :
     * @param token
     * @return: void
     **/
    public JSONArray  queryJobBuCondition(QueryJobDTO queryJobDTO, String token) {
        ResponseResult result = jobService.queryJobBuCondition(queryJobDTO,token);
        if(!BaseUtils.CheckResult(result)){
            throw new BusinessException("query role by name  error: " +BaseUtils.getMsg(result));
        }
        JSONArray roleArray = BaseUtils.getContent2Array(result);
         if(null == roleArray){
             throw new BusinessException("query role by name  error: no content info...");
         }
        return roleArray;
    }

    /**
     * @Description: 查询需要同步的用户信息
     * @Author: xgq
     * @Date: 2021/4/28 16:16

     * @return: java.util.List<com.persagy.sso.pojo.dto.WdUser>
     **/
    @DS("syn")
    public List<WdUser> querySynUserList() {
        LambdaQueryWrapper<WdUser> queryWrapper = new WdUser.Builder().createQueryWrapper().builderQueryWrapper();
        List<WdUser> userList = userService.list(queryWrapper);
        return userList;
    }


    /**
     * @Description: 根据来源查询账号信息
     * @Author: xgq     
     * @Date: 2021/4/29 10:20 
     * @param token :
     * @return: com.alibaba.fastjson.JSONObject
     **/
    public JSONArray queryAccountListBySource(String token) {
        AccountDTO accountDTO = new AccountDTO();
        accountDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
        accountDTO.setAccountSource(BaseConstant.ACCOUNT_SOURCE_WD);
        ResponseResult result  =   accountService.queryAccountListBySource(accountDTO,token);
        if(!BaseUtils.CheckResult(result)){
            throw new BusinessException("query account info by resource error:"+BaseUtils.getMsg(result) );
        }
        JSONArray array = BaseUtils.getContent2Array(result);
        if(null == array || (null !=array &&  array.size()==0)){
            return null;
        } else {
            return array;
        }
    }

    
    /**
     * @Description: 保存账号信息
     * @Author: xgq
     * @Date: 2021/4/29 10:49
     * @param user:
     * @param token:
     * @return: void
     **/
    @Transactional
    public void saveAccountItem(WdUser user, String token) {
        //保存账号信息
        SaasAccountCreateDTO createDTO = new SaasAccountCreateDTO();
        createDTO.setAccountBelong(BaseConstant.ACCOUNT_TYPE_BUSINESS);
        createDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
        createDTO.setAppId(BaseConstant.APP_ID_PC);
        createDTO.setUsername(user.getUsername());
        createDTO.setAccountType(BaseConstant.ACCOUNT_TYPE_NORMAL);
        //accountSource：万达：201
        createDTO.setAccountSource(BaseConstant.ACCOUNT_SOURCE_WD);
        //账号有效期，0-期内有效，1-永久有效
        createDTO.setValidLast(BaseConstant.VALID_TRUE);
        createDTO.setValid(Integer.valueOf(BaseConstant.VALID_TRUE));
      //  createDTO.setCasType(BaseConstant.CASE_TYPE_MAJOR);
     //   createDTO.setRoles(Arrays.asList(accountDTO));
        ResponseResult result =   accountService.insertAccount(createDTO,token);
        if (!BaseUtils.CheckResult(result)) {
            throw new RuntimeException("save account info error:"+BaseUtils.getMsg(result) );
        }
        //保存其他信息
        //账号id
        String accountId = BaseUtils.getContentStr(result);
        WdAccount wdAccount = new WdAccount();
        wdAccount.setAccountId(accountId);
        wdAccount.setEmployeeId(user.getEmployeeID());
        wdAccount.setEmployeeCode(user.getEmployeeCode());
        wdAccount.setEmployeeName(user.getEmployeeName());
        wdAccount.setGender(user.getGender());
        wdAccount.setUsername(user.getUsername());
        wdAccount.setUnitId(user.getUnitID());
        wdAccount.setUnitName(user.getUnitName());
        wdAccount.setJobId(user.getJobId());
        wdAccount.setJobName(user.getJobName());
        wdAccount.setEmployeeStatusName(user.getEmployeeStatusName());
        wdAccount.setEmployeeStatus(user.getEmployeeStatus());
        wdAccount.setStatus(user.getStatus());
        wdAccount.setCreateIme(new Date());
        wdAccount.setUpdateTime(new Date());
        boolean save = wdAccountService.save(wdAccount);
        if(!save){
            throw new BusinessException("save account reflex info error..." );
        }
    }



    public void updateAccountItem(WdAccount wdAccount, WdUser user, String token) {
        //重新赋值
        wdAccount.setEmployeeCode(user.getEmployeeCode());
        wdAccount.setEmployeeName(user.getEmployeeName());
        wdAccount.setGender(user.getGender());
        wdAccount.setUsername(user.getUsername());
        wdAccount.setUnitId(user.getUnitID());
        wdAccount.setUnitName(user.getUnitName());
        wdAccount.setJobId(user.getJobId());
        wdAccount.setJobName(user.getJobName());
        wdAccount.setEmployeeStatusName(user.getEmployeeStatusName());
        wdAccount.setEmployeeStatus(user.getEmployeeStatus());
        wdAccount.setStatus(user.getStatus());
        wdAccount.setUpdateTime(new Date());
        boolean save = wdAccountService.updateById(wdAccount);
        if(!save){
            throw new BusinessException("update account reflex info error..." );
        }
    }


    public List<WdAccount> queryAllReflexList() {
        return wdAccountService.list();
    }


    public void deleteSynAccount(Map<String, WdAccount> reflexAccountMap, String token) {
        //整理数据
        List<String> idList = new ArrayList<>();
        List<String> accountIdList = new ArrayList<>();
        reflexAccountMap.entrySet().stream().forEach(
                s->{
                    WdAccount value = s.getValue();
                    if(null !=value){
                        idList.add(value.getId());
                        accountIdList.add(value.getAccountId());
                    }
                }
        );
        //删除映射表的信息
        boolean delete = wdAccountService.removeByIds(idList);
        if(!delete){
            throw new BusinessException("delete account reflex info error..." );
        }
        //删除账号表的信息
        AccountDTO accountDTO = new AccountDTO();
        accountDTO.setAccountBelong(BaseConstant.ACCOUNT_TYPE_BUSINESS);
        accountDTO.setAccountType(BaseConstant.ACCOUNT_TYPE_NORMAL);
        accountDTO.setAccountSource(BaseConstant.ACCOUNT_SOURCE_WD);
        accountDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
        accountDTO.setAppId(BaseConstant.APP_ID_PC);
        accountDTO.setIds(accountIdList);
             ResponseResult  result = accountService.deleteByIds(accountDTO,token);
        if(!BaseUtils.CheckResult(result)){
            throw new BusinessException("delete account  info error..." );
        }
    }


    public void dealDeletedSynAccount(Map<String, WdAccount> reflexAccountMap, String token) throws Exception{
        //查询之前同步的账号信息
        JSONArray array = queryAccountListBySource(token);
        //如果查询的数量比映射表多，说明有垃圾数据没有清除掉
       if(null != array){
           int mapCount = reflexAccountMap.size();
           int arrayCount = array.size();
           if(arrayCount > mapCount){
               //获取多出来的数据删除
               List<String> exitAccountIdList = new ArrayList<>();
               reflexAccountMap.entrySet().stream().forEach(
                       s->{
                           WdAccount value = s.getValue();
                           if(null !=value){
                               exitAccountIdList.add(value.getAccountId());
                           }
                       }
               );
            // 获取需要删除的
               List<String> deleteAccountIdList = new ArrayList<>();
               for (int i = 0; i <array.size(); i++) {
                   JSONObject jsonObject = array.getJSONObject(i);
                   String id = jsonObject.getString("id");
                   if(!exitAccountIdList.contains(id)){
                       deleteAccountIdList.add(id);
                   }
               }
               if(CollectionUtils.isNotEmpty(deleteAccountIdList)){
                   //删除账号表的信息
                   AccountDTO accountDTO = new AccountDTO();
                   accountDTO.setAccountBelong(BaseConstant.ACCOUNT_TYPE_BUSINESS);
                   accountDTO.setAccountType(BaseConstant.ACCOUNT_TYPE_NORMAL);
                   accountDTO.setAccountSource(BaseConstant.ACCOUNT_SOURCE_WD);
                   accountDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
                   accountDTO.setAppId(BaseConstant.APP_ID_PC);
                   accountDTO.setIds(deleteAccountIdList);
                   ResponseResult  result = accountService.deleteByIds(accountDTO,token);
                   if(!BaseUtils.CheckResult(result)){
                       throw new BusinessException("delete account  info error..." );
                   }
               }
           }
       }
    }


    public void deleteAccountReflexById(String id) {
        boolean result = wdAccountService.removeById(id);
        if(!result){
            throw new BusinessException("delete account reflex info error ...");
        }
    }

    public JSONObject queryAccountById(String accountId ,String token) {
        AccountDTO accountDTO = new AccountDTO();
        accountDTO.setGroupCode(BaseConstant.GROUP_CODE_WD);
        accountDTO.setAccountSource(BaseConstant.ACCOUNT_SOURCE_WD);
        accountDTO.setAccountId(accountId);
        ResponseResult result  =   accountService.queryAccountById(accountDTO,token);
        if(!BaseUtils.CheckResult(result)){
            throw new BusinessException("query account info by resource error:"+BaseUtils.getMsg(result) );
        }
        JSONArray array = BaseUtils.getContent2Array(result);
        if(null == array || (null !=array &&  array.size()==0)){
            return null;
        } else {
            return array.getJSONObject(0);
        }
    }
}
