package com.persagy.sso.pojo.dto;

import com.persagy.common.constant.SaasCommonConstant;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

import javax.validation.constraints.NotBlank;
import java.util.Date;
import java.util.List;

/**
 * 项目信息
 * 
 * @version 1.0.0
 * @company xgq
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ApiModel
@ToString
public class SaasAccountCreateDTO {

	@ApiModelProperty(value = "主键ID")
	private String accountId; 	// 当前账号ID
	
	@ApiModelProperty(value = "关联类型，0-主岗，1-副岗，2-业务超管默认角色", example = "0")
	private String casType; 	// 关联类型，0-主岗，1-副岗，2-业务超管默认角色，这里关联类型会直接插入数据库中，不做为重置的条件
	
	@ApiModelProperty(value = "账号关联的角色ID集合")
	private List<AccountDTO> roles; 	// 账号关联的角色ID集合
	
	@ApiModelProperty(value = "集团编码")
	private String groupCode; 	// 所属集团编码
	
	@ApiModelProperty(value = "应用ID， PC-电脑端，APP-手机应用端", example = "PC")
    private String appId;
	
	@NotBlank(message = "登录用户名，不可为空")
	@ApiModelProperty(value = "登录用户名", example = "zhangsan")
	private String username; // 登录用户名
	
	@ApiModelProperty(value = "登录密码")
	private String password; // 登录密码

	@ApiModelProperty(value = "手机号", example = "18888888888")
	private String phoneNum; // 手机号
	
	@ApiModelProperty(value = "邮箱", example = "18888888888@163.com")
	private String mail; // 邮箱
	
	@ApiModelProperty(value = "头像，图片URL", example = "http://127.0.0.1:9980/test.img")
	private String headPortrait; // 头像，图片URL

	@ApiModelProperty(value = "账号类型, 0-超级管理员(所有集团)，1-单集团管理员(单集团)，2-普通账号", example = "0")
	private String accountType;
	
	@ApiModelProperty(value = "账号所属, 0-运维系统账号，1-业务账号", example = "0")
	private String accountBelong;	//账号所属, 0-运维系统账号，1-业务账号
	
	@ApiModelProperty(value = "账号来源", example = "persagy", required = true)
	private String accountSource = SaasCommonConstant.ACCOUNT_SOURCE_PERSAGE;

	@ApiModelProperty(value = "账号有效期的开始时间[时间戳：1615720476050]", example = "1615720476050")
	private Date validStartTime;

	@ApiModelProperty(value = "账号有效期的结束时间[时间戳：1615720476050]", example = "1615720476050")
	private Date validEndTime;

	@NotBlank(message = "账号有效期，不可为空")
	@ApiModelProperty(value = "账号有效期，0-期内有效，1-永久有效", example = "1",  required = true)
	private String validLast;	//账号有效期，0-期内有效，1-永久有效
	
	@ApiModelProperty(value = "备注", example = "测试账号")
	private String remark;
	
	@ApiModelProperty(value = "是否可用，0-不可用，1-可用", example = "1")
	private Integer valid;
	
	private boolean isVerify = true;
}
